unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Db, BTRVTables, Grids, DBGrids, StdCtrls, ExtCtrls, Spin,
  Buttons;

type
  TExecutionInfo = record
    Account : integer;
    Teller : smallint;
    Branch : smallint;
    AccountBalance : double;
    Delta : double;
    Transactional : boolean;
    Filler : string[145];
  end;
  TPanel = class(ExtCtrls.TPanel)
  private
    FLock: TRTLCriticalSection;
    Items : TStringList;
    NItems : integer;
    PER,PBR : TRect;
    FH : integer;
  protected
    procedure Resize; override;
    procedure Paint; override;
    procedure Loaded; override;
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure AddItem(const S : string; Advance : boolean);
  end;
  TForm1 = class(TForm)
    PageControl1: TPageControl;
    TCP_B: TTabSheet;
    Accounts: TBTRVTable;
    Branches: TBTRVTable;
    Tellers: TBTRVTable;
    History: TBTRVTable;
    TabSheet1: TTabSheet;
    PaintBox1: TPanel;
    ReadMe: TTabSheet;
    Panel1: TPanel;
    WhichReadMe: TRadioGroup;
    Panel2: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    WarmupTime: TSpinEdit;
    TestTime: TSpinEdit;
    CoolDownTime: TSpinEdit;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    Quiet: TCheckBox;
    Panel3: TPanel;
    Button2: TButton;
    Panel4: TPanel;
    Button4: TButton;
    Panel5: TPanel;
    Button3: TButton;
    Panel6: TPanel;
    GroupBox2: TGroupBox;
    Tables: TRadioGroup;
    Button1: TButton;
    Panel7: TPanel;
    NumAccounts: TRadioGroup;
    Panel8: TPanel;
    Panel9: TPanel;
    ReadmeBtn: TButton;
    RecordSelection: TRadioGroup;
    Memo1: TRichEdit;
    Memo2: TMemo;
    Panel10: TPanel;
    APT: TSpinEdit;
    StaticText1: TStaticText;
    StaticText2: TStaticText;
    TPB: TSpinEdit;
    Button5: TButton;
    Button6: TButton;
    Transactions: TRadioGroup;
    TabSheet2: TTabSheet;
    Panel11: TPanel;
    Panel12: TPanel;
    Panel13: TPanel;
    Button8: TButton;
    Label7: TLabel;
    DBDir: TEdit;
    SpeedButton1: TSpeedButton;
    Panel14: TPanel;
    MasterTable: TComboBox;
    Label8: TLabel;
    Label9: TLabel;
    DetailTable: TComboBox;
    Label10: TLabel;
    MasterLinkFlds: TEdit;
    Button7: TButton;
    Label11: TLabel;
    MasterIndex: TComboBox;
    Label12: TLabel;
    DetailIndex: TComboBox;
    TestDB: TBTRVDatabase;
    Test1: TBTRVTable;
    Test2: TBTRVTable;
    DSTest1: TDataSource;
    DSTest2: TDataSource;
    Button9: TButton;
    PageControl2: TPageControl;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    TabSheet5: TTabSheet;
    DBGrid1: TDBGrid;
    Splitter1: TSplitter;
    DBGrid2: TDBGrid;
    Panel15: TPanel;
    Filter: TEdit;
    Label13: TLabel;
    NoExtFilter: TCheckBox;
    BlockReadSize: TCheckBox;
    DBGrid3: TDBGrid;
    Splitter2: TSplitter;
    Memo3: TMemo;
    Button10: TButton;
    Button11: TButton;
    Button12: TButton;
    Panel16: TPanel;
    BLOB: TBTRVTable;
    DSBLOB: TDataSource;
    DBGrid4: TDBGrid;
    WhichBLOB: TRadioGroup;
    Button13: TButton;
    Panel17: TPanel;
    Button14: TButton;
    BLOB1Size: TLabel;
    BLOB2Size: TLabel;
    Button15: TButton;
    SaveDialog1: TSaveDialog;
    Label14: TLabel;
    NumThreads: TSpinEdit;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure ReadmeBtnClick(Sender: TObject);
    procedure WhichReadMeClick(Sender: TObject);
    procedure RecordSelectionClick(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure TransactionsClick(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure SetTestTable(Sender: TObject);
    procedure MasterIndexChange(Sender: TObject);
    procedure Button9Click(Sender: TObject);
    procedure TabSheet4Exit(Sender: TObject);
    procedure FilterSet(Sender: TObject);
    procedure Button10Click(Sender: TObject);
    procedure Button13Click(Sender: TObject);
    procedure Button14Click(Sender: TObject);
    procedure BLOBAfterScroll(DataSet: TDataSet);
    procedure Button15Click(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
  private
    InTest : boolean;
  public
    function GetTableFromTableList(TableList : TRadioGroup) : TBTRVTable;
    procedure ExecTest;
  end;

var
  Form1: TForm1;

implementation
{$R *.DFM}
uses BTRVConst, IniFiles, FileCtrl;

type
  TCountingThread = class(TThread)
  private
    ExecInfo : TExecutionInfo;
    Retries : integer;
    FLock: TRTLCriticalSection;
    Counting  : boolean;
    StartTime : cardinal;
    Count     : integer;
    Error     : string;
    Quiet     : boolean;

    DB        : TBTRVDatabase;
    History,
    Accounts,
    Tellers,
    Branches  : TBTRVTable;
  protected
    procedure Execute; override;
    procedure ReportError;
  public
    constructor Create(CreateSuspended : boolean);
    destructor Destroy; override;
    procedure StartCount;
    function EndCount(var Time : cardinal) : integer;
    function PerformTransaction : boolean;
  end;

constructor TCountingThread.Create(CreateSuspended : boolean);
begin
  inherited;
  Priority:=tpHighest;
  InitializeCriticalSection(FLock);
  Counting:=false;
  DB:=TBTRVDatabase.Create(nil); DB.TransactionType:=Form1.Accounts.Database.TransactionType;
  Accounts:=TBTRVTable.Create(nil); Accounts.Assign(Form1.Accounts); Accounts.Database:=DB; Accounts.Active:=true; Accounts.Name:='Accounts';
  Branches:=TBTRVTable.Create(nil); Branches.Assign(Form1.Branches); Branches.Database:=DB; Branches.Active:=true; Branches.Name:='Branches';
  Tellers :=TBTRVTable.Create(nil); Tellers.Assign(Form1.Tellers);   Tellers.Database:=DB;  Tellers.Active:=true;  Tellers.Name:='Tellers';
  History :=TBTRVTable.Create(nil); History.Assign(Form1.History);   History.Database:=DB;  History.Active:=true;  History.Name:='History';
  Resume;
end;

destructor TCountingThread.Destroy;
begin
  DeleteCriticalSection(FLock);
  Accounts.Free;
  Tellers.Free;
  Branches.Free;
  History.Free;
  DB.Free;
  inherited;
end;

procedure TCountingThread.Execute;
const
  Accounts_per_branch : integer = 10;
  Accounts_per_teller : integer = 10;
  Tellers_per_branch : integer = 1;
  NumTellers : integer = 10;
var
  PossibleBranches : integer;

  procedure SetFieldVals;
  var
    B : integer;
  begin
    with ExecInfo do begin
      Teller:=Random(NumTellers)+1;
      Branch:=Teller div Tellers_per_branch+1;
      Account:=Random(Accounts_per_branch)+1;
      B := Random(possibleBranches);
      if (B = Branch) and (possibleBranches>0) then Inc(B);
      Inc(Account,B*ACCOUNTS_PER_BRANCH);
      Delta:=999999-Random(1999998);
    end;
  end;

begin
  Randomize; Quiet:=Form1.Quiet.Checked;
  FillChar(ExecInfo,SizeOf(ExecInfo),0);
  PossibleBranches:=Branches.RecordCount-1; if PossibleBranches<0 then PossibleBranches:=0;
  NumTellers:=Tellers.RecordCount-1;
  Accounts_per_branch := Accounts.RecordCount div Branches.RecordCount;
  Accounts_per_teller := Accounts.RecordCount div Tellers.RecordCount;
  Tellers_per_branch := Accounts_per_branch div Accounts_per_teller;
  ExecInfo.Transactional:=Form1.Transactions.ItemIndex>0;

  while not Terminated do begin
    EnterCriticalSection(FLock);
    SetFieldVals;
    if Counting then Inc(Count);
    PerformTransaction;
    if not Quiet then begin
      Error:=IntToStr(Count)+' Account : '+IntToStr(ExecInfo.Account)+
                             ' Teller : '+IntToStr(ExecInfo.Teller)+
                             ' Branch : '+IntToStr(ExecInfo.Branch)+
                             ' Amount : '+FloatToStrF(ExecInfo.Delta,ffFixed,10,2);
      ReportError;
    end;
    LeaveCriticalSection(FLock);
  end;
end;

procedure TCountingThread.ReportError;
begin
  if Quiet then begin
    Form1.PaintBox1.AddItem(Error,false);
    Form1.PaintBox1.AddItem('',true)
  end else Form1.PaintBox1.AddItem(Error,true);
end;

procedure TCountingThread.StartCount;
begin
  EnterCriticalSection(FLock);
{
  while not TryEnterCriticalSection(FLock) do begin
    Form1.PaintBox1.AddItem('Cannot lock critical section',true);
    Application.ProcessMessages;
  end;
}
  Count:=0; Counting:=true;
  StartTime:=GetTickCount;
  LeaveCriticalSection(FLock);
end;

function TCountingThread.EndCount(var Time : cardinal) : integer;
begin
  EnterCriticalSection(FLock);
{
  while not TryEnterCriticalSection(FLock) do begin
    Form1.PaintBox1.AddItem('Cannot lock critical section',true);
    Application.ProcessMessages;
  end;
}
  Time:=GetTickCount-StartTime;
  Result:=Count;
  Counting:=false;
  LeaveCriticalSection(FLock);
end;

function TCountingThread.PerformTransaction : boolean;

  procedure UpdateTable(Tbl : TBTRVTable; Value : integer);
  var
    Konec : boolean;
  begin
    Konec:=true;
    repeat
      try
        Konec:=true;
        Inc(Retries);
        //Pazi, table se mora zakleniti, lock biasov pa nima implementiranih
        with Tbl do begin
          if not GetEqual([Value]) then raise Exception.Create(Name+': Value '+IntToStr(Value)+' not found');
          Edit;
          with Fields[2] do AsFloat:=AsFloat+ExecInfo.Delta;
          Post;
        end;
      except
        on E : Exception do begin
          if Tbl.Status<>78 then begin
            Error:=E.Message+' Retrying operation Retries : '+IntToStr(Retries);
            ReportError;
            Konec:=false;
          end else begin
            Error:=E.Message;
            ReportError;
            raise;
          end;
        end;
      end;
    until Konec;
  end;

  procedure InsertHistory;
  begin
    with ExecInfo do
      History.InsertRecord([Account,Teller,Branch,Delta,Now,'Filler']);
  end;

label RetryTrans;
begin
  RetryTrans:
  Result:=false;
  if ExecInfo.Transactional then
  try
    Tellers.Database.StartTransaction;
  except
    Exit;
  end;
  try
    Retries:=-3;
    UpdateTable(Branches,ExecInfo.Branch);
    UpdateTable(Accounts,ExecInfo.Account);
    UpdateTable(Tellers,ExecInfo.Teller);
    ExecInfo.AccountBalance:=Accounts.Fields[2].AsFloat;
    InsertHistory;
    if ExecInfo.Transactional then Tellers.Database.Commit;
    Result:=true;
  except
    if ExecInfo.Transactional then Tellers.Database.RollBack;
  end;
  if not Result and ((Accounts.Status=78) or (Tellers.Status=78) or (Branches.Status=78)) then begin
    Error:='Retrying whole transaction';
    ReportError;
    goto RetryTrans;
  end;
end;

procedure TForm1.ExecTest;
var
  Quiet : boolean;
  TCount: integer;
  Threads : TList;
  TotalTime,TotalCount : cardinal;

  procedure DoTest(Time : integer; Measure : boolean);
  var
    T1 : cardinal;

    function CheckTime : boolean;
    var
      j : cardinal;
    begin
      j:=GetTickCount;
      Result:=J>T1;
      if not Result and Quiet and ((T1-J) div 1000<cardinal(Time)) then begin
        Time:=(T1-J) div 1000;
        PaintBox1.AddItem('Remaining - '+IntToStr(Time)+' seconds',false);
      end;
    end;

  var
    i : integer;
    T : cardinal;
  begin
    if Measure then
      for i:=0 to TCount do
        TCountingThread(Threads[i]).StartCount;
    T1:=GetTickCount+cardinal(Time*1000);
    repeat
      Application.ProcessMessages;
      if CheckTime then break;
    until false;
    if Measure then begin
      TotalTime:=0; TotalCount:=0;
      for i:=0 to TCount do begin
        Inc(TotalCount,TCountingThread(Threads[i]).EndCount(T));
        Inc(TotalTime,T);
      end;
      TotalTime:=(TotalTime+cardinal(TCount) div 2) div cardinal(TCount+1);
    end;
  end;

var
  i : integer;
begin
  InTest:=true;
  try
    Accounts.Active:=false; Branches.Active:=false; Tellers.Active:=false; History.Active:=false;
    Accounts.Active:=true;
    Branches.Active:=true;
    Tellers.Active:=true;
    History.Active:=true;
    if Accounts.RecordCount=0 then begin
      PaintBox1.AddItem('Fill some accounts into the tables first',true);
      PaintBox1.AddItem('Go to File creation tab, select number',true);
      PaintBox1.AddItem('of accounts to fill and press "Fill accounts" button',true);
      Exit;
    end;
    Quiet:=Self.Quiet.Checked; TCount:=NumThreads.Value-1;
    Threads:=TList.Create;

    PaintBox1.AddItem('Warm up period - '+IntToStr(WarmupTime.Value)+' seconds',true); PaintBox1.AddItem('',true);
    for i:=0 to TCount do
      Threads.Add(TCountingThread.Create(true));

    DoTest(WarmupTime.Value,false);
    PaintBox1.AddItem('Steady state period - '+IntToStr(TestTime.Value)+' seconds',true); PaintBox1.AddItem('',true);
    DoTest(TestTime.Value,true);
    PaintBox1.AddItem('Cool down period - '+IntToStr(CoolDownTime.Value)+' seconds',true); PaintBox1.AddItem('',true);
    DoTest(CoolDownTime.Value,false);

    for i:=0 to TCount do
      with TCountingThread(Threads[i]) do begin
        Terminate;
        WaitFor;
        Free;
      end;

    PaintBox1.AddItem('Average elapsed time : '+FloatToStrF(TotalTime/(TotalCount*1000),ffFixed,10,5),true);
    PaintBox1.AddItem('Total elapsed time : '+FloatToStrF(TotalTime/1000,ffFixed,10,3),true);
    PaintBox1.AddItem('Number of iterations : '+FloatToStrF(TotalCount,ffFixed,10,0),true);
    PaintBox1.AddItem('',true);
    PaintBox1.AddItem('Transactions per second : '+FloatToStrF(TotalCount/(TotalTime/1000),ffFixed,10,1){+'   '+FloatToStrF((TotalCount/(TotalTime/100000))/575,ffFixed,10,1)+'% of native interface'},true);
    PaintBox1.AddItem('',true);
    PaintBox1.AddItem('Done!',true);
    PaintBox1.AddItem('',true);

    Accounts.Active:=false; Branches.Active:=false; Tellers.Active:=false; History.Active:=false;
    Accounts.Active:=true;
    Branches.Active:=true;
    Tellers.Active:=true;
    History.Active:=true;
  finally
    InTest:=false;
  end;
end;

constructor TPanel.Create(AOwner : TComponent);
begin
  inherited;
  InitializeCriticalSection(FLock);
  Items:=TStringList.Create;
end;

destructor TPanel.Destroy;
begin
  Items.Free;
  DeleteCriticalSection(FLock);
  inherited;
end;

procedure TPanel.Resize;
begin
  inherited;
  Canvas.Font.Assign(Font); Canvas.Brush.Color:=Color; Canvas.Brush.Style:=bsSolid;
  PBR:=ClientRect;
  InflateRect(PBR,-(BorderWidth+BevelWidth*(Ord(BevelInner<>bvNone)+Ord(BevelOuter<>bvNone))),
                  -(BorderWidth+BevelWidth*(Ord(BevelInner<>bvNone)+Ord(BevelOuter<>bvNone))));
  FH:=Abs(Font.Height)*14 div 10;
  NItems:=(PBR.Bottom-PBR.Top) div FH;
  PER:=PBR; PER.Top:=PER.Bottom-FH;
  while Items.Count<NItems do Items.Insert(0,' ');
  while Items.Count>NItems do Items.Delete(0);
end;

procedure TPanel.Paint;
var
  i,y : integer;
begin
  inherited;
  Canvas.Brush.Style:=bsSolid; Canvas.Brush.Color:=Color;
  y:=PER.Top;
  for i:=NItems-1 downto 0 do begin
    Canvas.TextOut(PER.Left,y,Items[i]);
    Dec(y,FH);
  end;
end;

procedure TPanel.Loaded;
begin
  inherited;
  Resize;
end;

procedure TPanel.AddItem(const S : string; Advance : boolean);
begin
  EnterCriticalSection(FLock);
  if Advance then begin
    Items.Add(S);
    if Items.Count>NItems then Items.Delete(0);
    ScrollWindowEx(Handle,0,-FH,@PBR,@PBR,0,nil,0)
  end else Items[NItems-1]:=S;
  Canvas.FillRect(PER);
  Canvas.TextOut(PER.Left,PER.Top,S);
  ValidateRect(Handle,@PBR);
  LeaveCriticalSection(FLock);
end;

function TForm1.GetTableFromTableList(TableList : TRadioGroup) : TBTRVTable;
begin
  case TableList.ItemIndex of
    0 : Result:=Accounts;
    1 : Result:=Branches;
    2 : Result:=Tellers;
    else Result:=History;
  end;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  GetTableFromTableList(Tables).AnalyzeRecord(true);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  ExecTest;
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  case WhichReadme.ItemIndex of
    0,1 : PageControl1.ActivePageIndex:=WhichReadme.ItemIndex;
    else begin
      PageControl1.ActivePageIndex:=2;
      PageControl2.ActivePageIndex:=WhichReadme.ItemIndex-2;
    end;
  end;
end;

procedure TForm1.ReadmeBtnClick(Sender: TObject);
begin
  case PageControl1.ActivePage.PageIndex of
    2 : WhichReadMe.ItemIndex:=PageControl2.ActivePage.PageIndex+2;
    else WhichReadMe.ItemIndex:=PageControl1.ActivePage.PageIndex;
  end;
  PageControl1.ActivePage:=ReadMe;
  WhichReadMe.OnClick(nil);
end;

procedure TForm1.WhichReadMeClick(Sender: TObject);
var
  S : string;
begin
  case WhichReadMe.ItemIndex of
    0 : S:='CREATION.RTF';
    1 : S:='TCPB.RTF';
    2 : S:='MASTDET.RTF';
    3 : S:='FILTERS.RTF';
    4 : S:='BLOB.RTF';
  end;
  Memo1.Lines.LoadFromFile(S);
end;

procedure TForm1.RecordSelectionClick(Sender: TObject);

  procedure ForcePageSize(Force,KeyOnly : boolean);
  begin
    Accounts.PageSize:=4096;
    Accounts.ForcePageSize:=Force;
    if KeyOnly then Accounts.FileFlags:=Accounts.FileFlags+[FKeyOnly]
    else Accounts.FileFlags:=Accounts.FileFlags-[FKeyOnly];
    Branches.PageSize:=4096;
    Branches.ForcePageSize:=Force;
    if KeyOnly then Branches.FileFlags:=Branches.FileFlags+[FKeyOnly]
    else Branches.FileFlags:=Branches.FileFlags-[FKeyOnly];
    Tellers.PageSize:=4096;
    Tellers.ForcePageSize:=Force;
    if KeyOnly then Tellers.FileFlags:=Tellers.FileFlags+[FKeyOnly]
    else Tellers.FileFlags:=Tellers.FileFlags-[FKeyOnly];
    History.PageSize:=512;
    History.ForcePageSize:=Force;
  end;

begin
  Button2.Enabled:=false;
  Button6.Enabled:=false;
//  Button6.Enabled:=false;
  Accounts.Active:=false;
  Branches.Active:=false;
  Tellers.Active:=false;
  History.Active:=false;
  Accounts.FileFlags:=Accounts.FileFlags-[BalancedIndex];
  Branches.FileFlags:=Branches.FileFlags-[BalancedIndex];
  Tellers.FileFlags:=Tellers.FileFlags-[BalancedIndex];
  case RecordSelection.ItemIndex of
    0 : begin
      Accounts.BtrvFields[3].Size:=84;
      Branches.BtrvFields[3].Size:=84;
      Tellers.BtrvFields[3].Size:=84;
      History.BTRVFields[5].Size:=22;
      ForcePageSize(true,false);
    end;
    1 : begin
      Accounts.BtrvFields[3].Size:=84;
      Branches.BtrvFields[3].Size:=84;
      Tellers.BtrvFields[3].Size:=84;
      History.BTRVFields[5].Size:=22;
      ForcePageSize(false,false);
    end;
    2 : begin
      Accounts.BtrvFields[3].Size:=88;
      Branches.BtrvFields[3].Size:=88;
      Tellers.BtrvFields[3].Size:=88;
      History.BTRVFields[5].Size:=25;
      ForcePageSize(true,false);
    end;
    3 : begin
      Accounts.BtrvFields[3].Size:=88;
      Branches.BtrvFields[3].Size:=88;
      Tellers.BtrvFields[3].Size:=88;
      History.BTRVFields[5].Size:=25;
      ForcePageSize(false,false);
    end;
    4 : begin
      Accounts.BTRVFields[3].Size:=78;  Accounts.FileFlags:=Accounts.FileFlags+[BalancedIndex];
      Branches.BTRVFields[3].Size:=78; Branches.FileFlags:=Branches.FileFlags+[BalancedIndex];
      Tellers.BTRVFields[3].Size:=78;  Tellers.FileFlags:=Tellers.FileFlags+[BalancedIndex];
      History.BTRVFields[5].Size:=25;
      ForcePageSize(false,true);
    end;
  end;
end;

procedure TForm1.Button5Click(Sender: TObject);

  procedure AddM2(S : string);
  begin
    Memo2.Lines.Add(S);
    Memo2.Update;
  end;

var
  Ini : TIniFile;
begin
  Accounts.Active:=false;
  Tellers.Active:=false;
  Branches.Active:=false;
  History.Active:=false;
  Accounts.CreateTable; AddM2('Accounts created');
  Branches.CreateTable; AddM2('Branches created');
  Tellers.CreateTable;  AddM2('Tellers created');
  History.CreateTable;  AddM2('History created');
  Accounts.Active:=true;
  Branches.Active:=true;
  Tellers.Active:=true;
  History.Active:=true;
  Button2.Enabled:=true;
  Button6.Enabled:=true;
//  Button6.Enabled:=true;
  Ini:=TIniFile.Create('.\DEMO.INI');
  Ini.WriteInteger('General','TableCreateMode',RecordSelection.ItemIndex);
  Ini.Free;
end;

procedure TForm1.FormCreate(Sender: TObject);
var
  Ini : TIniFile;
  N   : integer;
begin
  Ini:=TIniFile.Create('.\DEMO.INI');
  N:=Ini.ReadInteger('General','TableCreateMode',-1);
  if N<>-1 then RecordSelection.ItemIndex:=N;
  RecordSelection.OnClick(nil);
  if N=-1 then Button5.Click
  else begin
    Button2.Enabled:=true;
    Button6.Enabled:=true;
  end;
  Ini.Free;

  if not BLOB.Exists then BLOB.CreateTable; BLOB.Active:=true;
  Accounts.Active:=true;  Accounts.LockFlags:=[tfSingleLock,tfWait];
  Tellers.Active:=true;   Tellers.LockFlags:=[tfSingleLock,tfWait];
  Branches.Active:=true;  Branches.LockFlags:=[tfSingleLock,tfWait];
  History.Active:=true;   //History.LockFlags:=[tfSingleLock,tfWait];
end;

procedure TForm1.Button6Click(Sender: TObject);
var
  A,T,CA   : integer;
  NA,NT,NC : double;
  I,N      : integer;
  Tim      : cardinal;
  S        : string;
begin
  A:=APT.Value; T:=TPB.Value; Randomize;
  if Accounts.RecordCount>0 then begin
    A:=Accounts.RecordCount div Tellers.RecordCount;
    T:=Tellers.RecordCount div Branches.RecordCount;
    CA:=Accounts.RecordCount+1;
  end else CA:=1;
  case NumAccounts.ItemIndex of
    0 : N:=10000;
    1 : N:=100000;
    else N:=1000000;
  end;
  S:=Caption;
  Tim:=GetTickCount; NA:=0; NT:=0;
  for I:=0 to N-1 do begin
    if I mod 1000=0 then begin
      Caption:=IntToStr(i);
    end;
    NC:=Random(10000000);
    Accounts.InsertRecord([CA,(CA div A) div T+1,NC,'Filler']);
    if I mod A=0 then begin
      Tellers.InsertRecord([CA div A+1,((CA div A)+1) div T+1,NA,'Filler']);
      NA:=0;
    end;
    if I mod (A*T)=0 then begin
      Branches.InsertRecord([(CA div A) div T+1,(CA div A) div T+1,NT,'Filler']);
      NT:=0;
    end;
    NA:=NA+NC; NT:=NT+NC; Inc(CA);
  end;
  Tim:=GetTickCount-Tim; Inc(N,N div A+(N div A) div T);
  Memo2.Lines.Add('Total time : '+FloatToStrF(Tim/1000,ffFixed,10,3));
  Memo2.Lines.Add('Records added per second : '+FloatToStrF(N / (Tim/1000),ffFixed,10,3));
  Memo2.Lines.Add('Number of accounts in table : '+IntToStr(Accounts.RecordCount));
  Memo2.Lines.Add('Number of tellers in table : '+IntToStr(Tellers.RecordCount));
  Memo2.Lines.Add('Number of branches in table : '+IntToStr(Branches.RecordCount));
  Caption:=S;
end;

procedure TForm1.TransactionsClick(Sender: TObject);
begin
  Accounts.Database.TransactionFlags:=[tfSingleLock,tfWait];
  case Transactions.ItemIndex of
    0 : Accounts.Database.TransactionType:=ttExclusive;
    1 : Accounts.Database.TransactionType:=ttExclusive;
    2 : Accounts.Database.TransactionType:=ttConcurent;
  end;
end;

procedure TForm1.SpeedButton1Click(Sender: TObject);
var
  S : string;
begin
  S:=DBDir.Text;
  if SelectDirectory('Choose a directory','',S) then begin
    if AnsiLastChar(S)<>'\' then S:=S+'\';
    DBDir.Text:=S;
  end;
end;

procedure TForm1.Button7Click(Sender: TObject);
var
  SL : TStringList;
begin
  Test1.Active:=false; Test2.Active:=false;
  MasterTable.Items.Clear;
  DetailTable.Items.Clear;
  MasterIndex.Items.Clear;
  DetailIndex.Items.Clear;
  TestDB.DatabaseName:=DBDir.Text;
  if TestDB.Exists then TestDB.Connected:=true
  else Exit;
  SL:=TStringList.Create;
  TestDB.GetTablesList(SL);
  MasterTable.Items.Text:=SL.Text;
  DetailTable.Items.Text:=SL.Text;
  SL.Free;
end;

procedure TForm1.SetTestTable(Sender: TObject);
var
  D : TBTRVTable;
  C : TComboBox;
  S : string;
  i,j : integer;
begin
  Test2.MasterSource:=nil;
  Test2.MasterFields:='';
  if Sender=MasterTable then begin
    D:=Test1;
    C:=MasterIndex;
  end else begin
    D:=Test2;
    C:=DetailIndex;
  end;
  C.Items.Clear;
  D.Active:=false;
  D.TableName:=TComboBox(Sender).Text;
  D.Active:=true;
  C.Items.Add('(none)');
  for i:=0 to D.IndexDefs.Count-1 do begin
    S:=D.IndexDefs[i].Name+' (';
    for j:=0 to D.IndexDefs[i].Segments.Count-1 do
      S:=S+D.IndexDefs[i].Segments[j].Field+';';
    AnsiLastChar(S)^:=')';
    C.Items.Add(S);
  end;
  C.ItemIndex:=0;
end;

procedure TForm1.MasterIndexChange(Sender: TObject);
var
  D : TBTRVTable;
begin
  if Sender=MasterIndex then D:=Test1 else D:=Test2;
  if TComboBox(Sender).ItemIndex=0 then D.IndexName:=''
  else D.IndexName:=D.IndexDefs[TComboBox(Sender).ItemIndex-1].Name;
end;

procedure TForm1.Button9Click(Sender: TObject);
begin
  Test2.MasterSource:=DSTest1;
  Test2.MasterFields:=MasterLinkFlds.Text;
end;

procedure TForm1.TabSheet4Exit(Sender: TObject);
begin
  Test1.Filtered:=false;
  Test1.Filter:='';
end;

procedure TForm1.FilterSet(Sender: TObject);
begin
  if Sender=Button11 then begin
    Test1.Filter:=Filter.Text;
    Test1.Filtered:=Test1.Filter<>'';
  end else begin
    Test1.Filter:='';
    Test1.Filtered:=false;
  end;
end;

procedure TForm1.Button10Click(Sender: TObject);
var
  T,i : cardinal;
  S,F,L : string;
begin
  F:=Test1.ExtendedFilter;
  if NoExtFilter.Checked then begin
    S:=' ';
    Test1.ExtendedFilter:='';
  end else S:='F';
  if BlockReadSize.Checked then begin
    Test1.BlockReadSize:=100;
    S:=S+'B ';
  end else S:=S+'  ';
  Test1.DisableControls;
  T:=GetTickCount; i:=0;
  Test1.First;
  while not Test1.EOF do begin
    Inc(i);
    Test1.Next;
  end;
  T:=GetTickCount-T;
  S:=S+'Records : ';
  L:=IntToStr(i);
  while Length(L)<6 do L:=' '+L;
  S:=S+L+' Time : ';
  L:=FloatToStrF(T/1000,ffNumber,10,3);
  while Length(L)<10 do L:=' '+L;
  S:=S+L;
  Memo3.Lines.Add(S);
  Test1.BlockReadSize:=0;
  Test1.ExtendedFilter:=F;
  Test1.EnableControls;
end;

procedure TForm1.Button13Click(Sender: TObject);
var
  OD : TOpenDialog;
begin
  if BLOB.State=dsBrowse then BLOB.Edit;
  OD:=TOpenDialog.Create(nil);
  if OD.Execute and (OD.FileName<>'') then begin
    case WhichBLOB.ItemIndex of
      0 : TBLOBField(BLOB.FieldByName('BLOB1')).LoadFromFile(OD.FileName);
      1 : TBLOBField(BLOB.FieldByName('BLOB2')).LoadFromFile(OD.FileName);
    end;
  end;
  OD.Free;
end;

procedure TForm1.Button14Click(Sender: TObject);
begin
  Close;
end;

procedure TForm1.BLOBAfterScroll(DataSet: TDataSet);
begin
  BLOB1Size.Caption:='BLOB 1 size : '+FormatFloat('###,###,###,###,##0',TBlobField(BLOB.FieldByName('BLOB1')).BlobSize)+' bytes';
  BLOB2Size.Caption:='BLOB 2 size : '+FormatFloat('###,###,###,###,##0',TBlobField(BLOB.FieldByName('BLOB2')).BlobSize)+' bytes';
end;

procedure TForm1.Button15Click(Sender: TObject);
var
  SD : TSaveDialog;
begin
  SD:=TSaveDialog.Create(nil); SD.Options:=SD.Options+[ofOverWritePrompt,ofPathMustExist,ofNoReadOnlyReturn];
  if SD.Execute and (SD.FileName<>'') then begin
    case WhichBLOB.ItemIndex of
      0 : TBLOBField(BLOB.FieldByName('BLOB1')).SaveToFile(SD.FileName);
      1 : TBLOBField(BLOB.FieldByName('BLOB2')).SaveToFile(SD.FileName);
    end;
  end;
end;

procedure TForm1.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  CanClose:=not InTest;
end;

end.
